

/* ***********************************************************************
                Naval Air Warfare Center / Aircraft Division              
                        Patuxent Naval Air Station                         
 ========================================================================

   File:           player.c                                         
                                                                        
   Classification: UNCLASSIFIED                                   
                                                                        
   Description:    This file contains the functions, methods, and global
      variables needed to extract and manipulate players defined in the TDB
			
   Written by:     Alex Harper
*********************************************************************** */


#include "parse_tdb.h"


/* ***********************************************************
  This method extracts data from the PLAYER block

   FILE *infile,                       // input file
   char *token,                        // last processed token
   int *errorflag                      // error flag
   dbl_llist_manager *clist,           // list of extracted capabilities
   dbl_llist_manager *susceptibility   // list of extracted susceptibilities
*********************************************************** */
void player_type::extract_tdbdat(FILE *infile, char *token, int *errorflag, dbl_llist_manager *clist, dbl_llist_manager *susceptibility, dbl_llist_manager *semcode_table) {
      
   platform_type *ptr;
   linkage_type *link;
           
   *errorflag = get_sweg_token(infile, token);	// player name
   name.stringcpy(token);

   *errorflag = get_sweg_token(infile, token);

   // extract platforms
   while (*errorflag) {

      if (!strcmp(token, TOKEN_PLATFORM)) {
         plat_man.append(ptr = new platform_type(this), NULL);
         ptr->extract_tdbdat(infile, token, errorflag, clist, susceptibility, semcode_table);
         continue;
      }

      if (!strcmp(token, TOKEN_LINKAGES))
         break;
	 
      if (!strcmp(token, TOKEN_PLAYER_STRUCTURE))
         break;
	 
      *errorflag = get_sweg_token(infile, token);
   }

   // extract linkage data
   if (plat_man.head && !strcmp(token, TOKEN_LINKAGES)) {

      *errorflag = get_sweg_token(infile, token);

      while (*errorflag) {

         if (!strcmp(token, TOKEN_END))
            break;
	    
         link = new linkage_type;
         link->first = atoi(token);
	 
         *errorflag = get_sweg_token(infile, token);	// "with"
         *errorflag = get_sweg_token(infile, token);
         link->second = atoi(token);	 

         link->firstptr = ((platform_type *)plat_man.head)->find_radar(link->first);
         link->secondptr = ((platform_type *)plat_man.head)->find_radar(link->second);

         if (!link->firstptr || !link->secondptr)
            delete link;
         else
            linkage_man.insert(link, NULL);
	 
         *errorflag = get_sweg_token(infile, token);
      }

      *errorflag = get_sweg_token(infile, token);	// PLAYER-STRUCTURE
   }
   
};


/* ***********************************************************
   This method dumps the PLAYER data to stdout

   name2sem *semcode_table,            // semantic code to platform name lookup table
   dbl_llist_manager *susceptibility,  // target susceptibility list
   char *prefix                        // tab delimited prefix
*********************************************************** */
void player_type::dump_data(dbl_llist_manager *susceptibility, char *prefix) {

   platform_type *ptr;
   char buffer[MAXSTRLEN];
   
   printf("%s%s\n", prefix, name.string);
   sprintf(buffer, "%s   ", prefix);
   
   for (ptr=(platform_type *)plat_man.head; ptr; ptr=(platform_type *)ptr->next)
      ptr->dump_data(susceptibility, buffer);
}


/* ***********************************************************
  This method cleans up any useless nodes
*********************************************************** */
void player_type::data_cleanup() {

   platform_type *ptr, *qtr;
   linkage_type *ltr, *mtr;
   radar_type *rec;

   // cleanup platforms  
   ptr = (platform_type *)plat_man.head;
   
   while (ptr) {
      qtr = ptr;
      ptr = (platform_type *)ptr->next;

      qtr->data_cleanup();
      if (!qtr->query_justify()) {
         plat_man.remove(qtr);
         delete qtr;
      }
	 
   }

   // cleanup links
   if (!plat_man.head) {
      linkage_man.dest();
      return;
   }
   
   ltr = (linkage_type *)linkage_man.head;
   
   while (ltr) {

      mtr = ltr;
      ltr = (linkage_type *)ltr->next;

      if (!((platform_type *)plat_man.head)->find_radar(mtr->first) || !((platform_type *)plat_man.head)->find_radar(mtr->second)) {
         linkage_man.remove(mtr);
         delete mtr;
      }

   }
         	          
   // init receivers
   for (ptr=(platform_type *)plat_man.head; ptr; ptr=(platform_type *)ptr->next)
      for (rec=(radar_type *)ptr->rec.head; rec; rec=(radar_type *)rec->next)
         rec->init_receiver(&linkage_man);
}


/* ***********************************************************
  This method determines if enough data is present for this
  player for radar calcs.
*********************************************************** */
int player_type::query_justify() {

   return (plat_man.head) ? 1 : 0;
}


/* ***********************************************************
  This method searches for the link between the EM system and
  its counterpart (transmitter and receiver)

  radar_type *radar  // find a linkage to this transmitter/receiver
*********************************************************** */
linkage_type *player_type::find_link(radar_type *radar) {

   linkage_type *link;
   
   for (link=(linkage_type *)linkage_man.head; link; link=(linkage_type *)link->next)
      if (link->firstptr == radar || link->secondptr == radar)
         return link;

   return NULL;	 
}
